@@ -78,9 +78,30 @@ attribute_map = {
 
 # order does really matter.
 # It is used to define ordering between the respected platforms
-platform_list = ["HSW", "BDW", "CHV", "SKL", "BXT", "KBL",
-        "GLK", "CNL", "ICL", "ICLLP", "TGLLP", "RKL", "DG1",
-        "ADLP", "ADLS", "XEHP", "DG2", "PVC", "PVCXT"]
+platform_list = [
+    "HSW",
+    "BDW",
+    "CHV",
+    "SKL",
+    "BXT",
+    "KBL",
+    "GLK",
+    "CNL",
+    "ICL",
+    "ICLLP",
+    "TGLLP",
+    "RKL",
+    "DG1",
+    "ADLP",
+    "ADLS",
+    "ADLN",
+    "XEHP",
+    "DG2",
+    "MTL",
+    "PVC",
+    "PVCXT_A0",
+    "PVCXT",
+]
 
 def getAttributeList(Attrs):
     """
