@@ -22,6 +22,7 @@ SPDX-License-Identifier: MIT
 #include "llvm/IR/Module.h"
 #include "llvm/Pass.h"
 
+#include "llvmVCWrapper/Analysis/InstructionSimplify.h"
 #include "llvmVCWrapper/IR/Attributes.h"
 #include "llvmVCWrapper/IR/DerivedTypes.h"
 #include "llvmVCWrapper/IR/Function.h"
@@ -201,20 +202,24 @@ static Value *createVectorToScalarValue(Value *Vector,
                                         Instruction *InsertBefore,
                                         size_t idx = 0) {
   assert(hasSingleElementVector(Vector->getType()));
+  Instruction *Val = nullptr;
   if (isa<UndefValue>(Vector))
     return UndefValue::get(
         getTypeFreeFromSingleElementVector(Vector->getType()));
   else if (isa<PointerType>(Vector->getType()))
-    return new BitCastInst(
-        Vector, getTypeFreeFromSingleElementVector(Vector->getType()),
-        "sev.cast.", InsertBefore);
+    Val = new BitCastInst(Vector,
+                          getTypeFreeFromSingleElementVector(Vector->getType()),
+                          "sev.cast.", InsertBefore);
   else if (auto *Const = dyn_cast<Constant>(Vector))
     return Const->getAggregateElement(idx);
   else {
     auto *M = InsertBefore->getModule();
-    return ExtractElementInst::Create(Vector, getVectorIndex(*M, idx),
-                                      "sev.cast.", InsertBefore);
+    Val = ExtractElementInst::Create(Vector, getVectorIndex(*M, idx),
+                                     "sev.cast.", InsertBefore);
   }
+  if (auto *InVector = dyn_cast<Instruction>(Vector))
+    Val->setDebugLoc(InVector->getDebugLoc());
+  return Val;
 }
 
 // This util accepts SEV-rich Value and returns new, SEV-free one
@@ -224,20 +229,24 @@ static Value *createVectorToScalarValue(Value *Vector,
 static Value *createVectorToScalarValue(Value *Vector, BasicBlock *BB,
                                         size_t idx = 0) {
   assert(hasSingleElementVector(Vector->getType()));
+  Instruction *Val = nullptr;
   if (isa<UndefValue>(Vector))
     return UndefValue::get(
         getTypeFreeFromSingleElementVector(Vector->getType()));
   else if (isa<PointerType>(Vector->getType()))
-    return new BitCastInst(
-        Vector, getTypeFreeFromSingleElementVector(Vector->getType()),
-        "sev.cast.", BB);
+    Val = new BitCastInst(Vector,
+                          getTypeFreeFromSingleElementVector(Vector->getType()),
+                          "sev.cast.", BB);
   else if (auto *Const = dyn_cast<Constant>(Vector))
     return Const->getAggregateElement(idx);
   else {
     auto *M = BB->getModule();
-    return ExtractElementInst::Create(Vector, getVectorIndex(*M, idx),
-                                      "sev.cast.", BB);
+    Val = ExtractElementInst::Create(Vector, getVectorIndex(*M, idx),
+                                     "sev.cast.", BB);
   }
+  if (auto *InVector = dyn_cast<Instruction>(Vector))
+    Val->setDebugLoc(InVector->getDebugLoc());
+  return Val;
 }
 
 // This util accepts Scalar Value and returns new SEV-rich Value
@@ -844,7 +853,7 @@ static void collapseBitcastInst(BitCastInst *BitCast, bool CollapseCannotFail) {
   }
   auto &&M = *BitCast->getModule();
   auto &&Q = SimplifyQuery(M.getDataLayout());
-  auto *ReplaceWith = SimplifyCastInst(
+  auto *ReplaceWith = VCINTR::SimplifyCastInst(
       BitCast->getOpcode(), BitCast->getOperand(0), BitCast->getType(), Q);
   if (!CollapseCannotFail && !ReplaceWith)
     return;
@@ -885,8 +894,8 @@ static void collapseExtractInst(ExtractElementInst *Extract,
   }
   auto &&M = *Extract->getModule();
   auto &&Q = SimplifyQuery(M.getDataLayout());
-  auto *ReplaceWith = SimplifyExtractElementInst(Extract->getOperand(0),
-                                                 Extract->getOperand(1), Q);
+  auto *ReplaceWith = VCINTR::SimplifyExtractElementInst(
+      Extract->getOperand(0), Extract->getOperand(1), Q);
   if (!CollapseCannotFail && !ReplaceWith)
     return;
   assert(ReplaceWith && "Oops... Cannot collapse ExtractElement instruction");
@@ -902,7 +911,7 @@ static void collapseInsertInst(InsertElementInst *Insert,
   }
   auto &&M = *Insert->getModule();
   auto &&Q = SimplifyQuery(M.getDataLayout());
-  auto *ReplaceWith = SimplifyInsertElementInst(
+  auto *ReplaceWith = VCINTR::SimplifyInsertElementInst(
       Insert->getOperand(0), Insert->getOperand(1), Insert->getOperand(2), Q);
 
   // SimplifyInsertElementInst provides too simple analysis
